SystemOrganization addCategory: #GraphAlgorithms!

TestCase subclass: #GraphAlgorithmsTest
	instanceVariableNames: 'nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GraphAlgorithms'!

!GraphAlgorithmsTest methodsFor: 'running' stamp: 'Deniels 12/21/2011 21:03'!
setUp 
	|view|
	view := MOViewRenderer new.
	nodes :=view nodes: (1 to: 6).
	view edgesFromAssociations: {
		"the node 3 is free"
		"the node 1 has more than others outgoingEdges"
		"the node 5 has more than others incomingEdges"
		1 -> 2.
		1 -> 4.
		2 -> 5.
		1 -> 5
	}.! !

!GraphAlgorithmsTest methodsFor: 'tests' stamp: 'Deniels 12/20/2011 16:31'!
testGraphFindFreeNodes 
 	| res |
	res := GraphFindFreeNodes new applyOn: nodes.
	self assert: res asArray = { nodes third . nodes sixth }! !

!GraphAlgorithmsTest methodsFor: 'tests' stamp: 'Deniels 12/21/2011 22:00'!
testGraphFindMostFanInNode
	| res |
	res := GraphFindMostFanInNodes new applyOn: nodes.
	self assert: res asArray = { nodes fifth }! !

!GraphAlgorithmsTest methodsFor: 'tests' stamp: 'Deniels 12/21/2011 22:00'!
testGraphFindMostFanOutNode
	| res |
	res := GraphFindMostFanOutNodes new applyOn: nodes.
	self assert: res asArray = { nodes first }
	! !

TestCase subclass: #GraphAlgorithmsTest2
	instanceVariableNames: 'nodes startNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GraphAlgorithms'!

!GraphAlgorithmsTest2 methodsFor: 'running' stamp: 'Deniels 12/21/2011 21:54'!
setUp 
	"No free nodes
	the node 1 and 2 has more than others outgoingEdges
	the node 2 has more than others incomingEdges"

	|view|
	view := MOViewRenderer new.
	nodes :=view nodes: (1 to: 6).
	startNode:= nodes third.
	view edgesFromAssociations: {
		1 -> 2.
		1 -> 6.
		6 -> 2.
		2 -> 3.
		2 -> 4.
		4 -> 5
	}.! !

!GraphAlgorithmsTest2 methodsFor: 'tests' stamp: 'Deniels 12/21/2011 20:57'!
testGraphFindFreeNodes 
 	| res |
	res := GraphFindFreeNodes new applyOn: nodes.
	self assert: res asArray = { }! !

!GraphAlgorithmsTest2 methodsFor: 'tests' stamp: 'Deniels 12/21/2011 22:00'!
testGraphFindMostFanInNode
	| res |
	res := GraphFindMostFanInNodes new applyOn: nodes.
	self assert: res asArray = { nodes second }! !

!GraphAlgorithmsTest2 methodsFor: 'tests' stamp: 'Deniels 12/21/2011 22:00'!
testGraphFindMostFanOutNode
	| res |
	res := GraphFindMostFanOutNodes new applyOn: nodes.
	self assert: res asArray = { nodes first . nodes second}
	! !

!GraphAlgorithmsTest2 methodsFor: 'tests' stamp: 'Deniels 12/22/2011 00:00'!
testGraphFindTheLongestPath
	| res |
	res := GraphFindTheLongestPath new applyOn: nodes startNode: startNode.
	"self assert: res asArray = { nodes first . nodes sixth . nodes second . nodes fourth . nodes fifth}"
	Transcript show: res.
	self assert: res asArray = { nodes third}! !

Object subclass: #GraphAlgorithms
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GraphAlgorithms'!

!GraphAlgorithms methodsFor: 'public - hooks' stamp: 'Deniels 12/21/2011 21:16'!
applyOn: nodes! !

!GraphAlgorithms methodsFor: 'public - hooks' stamp: 'Deniels 12/21/2011 21:18'!
applyOn: nodes startNode: startNode! !

!GraphAlgorithms methodsFor: 'public - hooks' stamp: 'Deniels 12/20/2011 16:29'!
highlight: nodes
	self applyOn: nodes.
	! !

GraphAlgorithms subclass: #GraphFindFreeNodes
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GraphAlgorithms'!

!GraphFindFreeNodes methodsFor: 'public - hooks' stamp: 'Deniels 12/20/2011 16:31'!
applyOn: nodes
	"find all free nodes (i.e., a node that does not have an icoming and outgoing edge)"

	^ nodes select: [ :n | (n incomingEdges isEmpty) and: [ n outgoingEdges isEmpty ] ].
	! !

!GraphFindFreeNodes methodsFor: 'public - hooks' stamp: 'Deniels 12/20/2011 16:50'!
highlight: nodes
	(self applyOn: nodes) do: [ :node | node copyShapeAndDo: [ :shape | shape fillColor: Color red ]]! !

GraphAlgorithms subclass: #GraphFindMostFanInNodes
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GraphAlgorithms'!

!GraphFindMostFanInNodes methodsFor: 'public - hooks' stamp: 'Deniels 12/21/2011 21:07'!
applyOn: nodes
	"find a node that has the most incomingEdges"
	
	|maxFanIn theNodes|
	maxFanIn := 0.
	theNodes := OrderedCollection new.
	nodes do:[:n | (n incomingEdges size > maxFanIn) ifTrue:[theNodes reset. theNodes add: n. maxFanIn := n incomingEdges size] ifFalse:[(n incomingEdges size = maxFanIn) ifTrue: [theNodes add: n. maxFanIn := n incomingEdges size]]].
	^theNodes! !

!GraphFindMostFanInNodes methodsFor: 'public - hooks' stamp: 'Deniels 12/20/2011 22:35'!
highlight: nodes
	(self applyOn: nodes) do: [ :node | node copyShapeAndDo: [ :shape | shape fillColor: Color red ]]! !

GraphAlgorithms subclass: #GraphFindMostFanOutNodes
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GraphAlgorithms'!

!GraphFindMostFanOutNodes methodsFor: 'public - hooks' stamp: 'Deniels 12/21/2011 21:10'!
applyOn: nodes
	"find a node that has the most incomingEdges"
	
	|maxFanOut theNodes|
	maxFanOut := 0.
	theNodes := OrderedCollection new.
	nodes do:[:n | (n outgoingEdges size > maxFanOut) ifTrue:[theNodes reset. theNodes add: n. maxFanOut := n outgoingEdges size] ifFalse:[(n outgoingEdges size = maxFanOut) ifTrue: [theNodes add: n. maxFanOut := n outgoingEdges size]]].
	^theNodes! !

!GraphFindMostFanOutNodes methodsFor: 'public - hooks' stamp: 'Deniels 12/20/2011 22:41'!
highlight: nodes
	(self applyOn: nodes) do: [ :node | node copyShapeAndDo: [ :shape | shape fillColor: Color red ]]! !

GraphAlgorithms subclass: #GraphFindTheLongestPath
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GraphAlgorithms'!

!GraphFindTheLongestPath methodsFor: 'public - hooks' stamp: 'Deniels 12/22/2011 00:02'!
applyOn: nodes startNode: startNode
	"find the longest path form startNode within a grahp"
	|longestPath path|
	path := OrderedCollection new.
	longestPath := OrderedCollection new.
	(startNode outgoingEdges isEmpty) ifTrue: [^ longestPath add: (path add: startNode)].
	((startNode outgoingEdges) collect: #target) do: [:node | path add: startNode. self applyOn: nodes startNode: node]
	
	"No alcance a verificar si esta bueno, aunque no lo creo. La idea era encontrar todas las rutas desde el startNode hasta las hojas y despues retornar el arreglo mas largo (path), dentro de longestPath"! !

Object subclass: #ReadMe
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GraphAlgorithms'!

!ReadMe methodsFor: 'as yet unclassified' stamp: 'Deniels 12/22/2011 00:03'!
readMe
	"Project Name: Graph Algorithm in Mondrian
		This project is to implement a serie of algorithms to identify certain characteristics in graphos 
		so far the algorithms implemented are FindFreeNodes , FindMostFanInNodes and FindMostFanOutNodes. 
		All classes have the methods applayOn and highlight. 
		applayOn, receives as parameter all nodes in the graph, and return an array with all nodes that satisfy the 
		method's condition.
		The highlight method take the nodes found and paint it
		example of use:
		write in workspace:
	|view nodes|
	view := MOViewRenderer new.
	view shape rectangle
		size: 20.
	nodes := view nodes: (1 to: 6).
	view edgesFromAssociations: {
		1 -> 2.
		1 -> 6.
		6 -> 2.
		2 -> 3.
		2 -> 4.
		4 -> 5
	}.
	GraphFindMostFanOutNodes new highlight: nodes.
	view treeLayout.
	view open"! !
